

import java.io.File;

import com.logicaldoc.webservice.document.WSDocument;
import com.logicaldoc.webservice.rest.auth.AuthClient;
import com.logicaldoc.webservice.rest.document.DocumentClient;

public class RestWorkbench {

	public static void mainCheckin(String[] args) throws Exception {

		/**
		 * Cross-check the file size before and after the checkin
		 * Use the new REST web-service for the check in the file.
		 * The special feature of the REST API (LD 7.0.x) check-in method is to allow the upload of large files
		 */
		
		String base = "http://localhost:8080/services/rest";
		String soapBase = "http://localhost:8080/services/";

		AuthClient auth = new AuthClient(base + "/auth");
		DocumentClient doc = new DocumentClient(base + "/document");
		com.logicaldoc.webservice.document.DocumentClient soapclient = new com.logicaldoc.webservice.document.DocumentClient(soapBase + "/Document?wsdl");

		// Open a session
		String sid = auth.login("admin", "admin");
		System.out.println("Created session: " + sid);
		if (auth.valid(sid))
			System.out.println("Session is valid");

		try {
			long docID = 16416768L;
			doc.checkout(sid, docID);
			System.out.println("Checkout performed");
			
			File file = new File("C:/tmp/logicaldoc-dropbox.m2v");
			// get the filesize in order to check
			long fsize = file.length();
			System.out.println("FileSize of the new file version: " +fsize);						
			
			//checkin(String sid, long docId, String comment, String filename, boolean release, File file)
			doc.checkin(sid, docID, "rest checkin", file.getName(), false, file);								
			System.out.println("Operation completed");
			
			// Checks the filesize after the check-in operation (using the SOAP ws)
			WSDocument xxx = soapclient.getDocument(sid, docID);
			System.out.println("FileSize after checkin: " + xxx.getFileSize());
		} finally {
			auth.logout(sid);
		}
	}
	
	
	public static void main(String[] args) throws Exception {

		/**
		 * Performs the creation of a new document using the REST API (LD 7.0.x).
		 * The special feature of the REST API upload method allows the upload of large files
		 */
		
		String base = "http://localhost:8080/services/rest";
		String soapBase = "http://localhost:8080/services/";

		AuthClient auth = new AuthClient(base + "/auth");
		DocumentClient restClient = new DocumentClient(base + "/document");
		com.logicaldoc.webservice.document.DocumentClient soapclient = new com.logicaldoc.webservice.document.DocumentClient(soapBase + "/Document?wsdl");

		// Open a session
		String sid = auth.login("admin", "admin");
		System.out.println("Created session: " + sid);
		if (auth.valid(sid))
			System.out.println("Session is valid");

		try {
			File file = new File("C:/tmp/logicaldoc-dropbox.m2v");
			if (!file.exists()) {
				throw new Exception("upload File not found!");
			}
			
			// get the filesize in order to check
			long fsize = file.length();
			System.out.println("FileSize of the new file: " +fsize);								

			long folderID = 4L; // main document folder
			//upload(String sid, Long docId, Long folderId, boolean release, String filename, File file)
			long docID = restClient.upload(sid, null, folderID, true, file.getName(), file);
										
			System.out.println("Operation completed");
			System.out.println("Document created with ID: " + docID);
			
			// Checks the filesize after the upload operation (using the SOAP ws) [OPTIONAL]
			WSDocument xxx = soapclient.getDocument(sid, docID);
			System.out.println("FileSize after upload: " + xxx.getFileSize());
		} finally {
			auth.logout(sid);
		}
	}
	

	public static void mainAuth(String[] args) throws Exception {
		
		/**
		 * Performs authentication using the REST api
		 */
	
		String base = "http://localhost:8080/services/rest";
		AuthClient auth = new AuthClient(base + "/auth");

		// Open a session
		String sid = auth.login("admin", "admin");
		System.out.println("Created session: " + sid);
		if (auth.valid(sid))
			System.out.println("Session is valid");

		auth.renew(sid);
		System.out.println("Session renewed");

		try {
		} finally {
			auth.logout(sid);
		}
	}
}
