package org.logicaldoc.test.ws;

import java.io.File;
import java.text.SimpleDateFormat;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.logicaldoc.webservice.document.DocumentClient;
import com.logicaldoc.webservice.document.WSDocument;

public class TestDocumentCRUD extends BaseTestCase {
	
	protected static Log log = LogFactory.getLog(TestDocumentCRUD.class);
	
	public TestDocumentCRUD(String arg0) {
		super(arg0);
	}

	public void testDocumentCRUD() throws Exception {

		DocumentClient docc = new DocumentClient(DOC_ENDPOINT);

		File file = new File("C:/tmp/2013-281-out.pdf");

		DataSource ds = new FileDataSource(file);
		DataHandler content = new DataHandler(ds);

		WSDocument document = new WSDocument();
		document.setTitle("docTitle");
		document.setLanguage("en");
		document.setFolderId(DEFAULT_WORKSPACE);
		document.setFileName(file.getName());
		document.setSourceAuthor("sourceAuthor");
		
		WSDocument docRes = docc.create(sid, document, content);
		

		System.err.println("documentID = " + docRes.getId());
		
		// Retrieve the metadata of the document created 
		long docId = docRes.getId();
		WSDocument docInfo = docc.getDocument(sid, docId);
		
		System.out.println(docInfo.getFileName());
		System.out.println(docInfo.getTitle());
		System.out.println(docInfo.getSourceDate());
		
		assertEquals("sourceAuthor", docInfo.getSourceAuthor());
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String currentDateStr = sdf.format(new java.util.Date());
		
		assertTrue(docInfo.getSourceDate().startsWith(currentDateStr));
		
		// Update the metadata of the document created
		// in particular we decided to update fields: title, source, sourceDate, sourceAuthor, tags
		try {
			String[] tags = new String[]{"tag", "keyword", "test"};
			
			docInfo.setTitle("MyTitle");
			docInfo.setSource("MySource");
			docInfo.setSourceAuthor("MysourceAuthor");
			docInfo.setSourceDate("2012-04-12");
			docInfo.setTags(tags);
			docc.update(sid, docInfo);
					
		} catch (RuntimeException e1) {
			e1.printStackTrace();
		}

		
		//VERIFY THE CHANGES to the tags
		WSDocument docInfoUpd = docc.getDocument(sid, docId);
		String[] tags2 = docInfoUpd.getTags();
		assertNotNull(tags2);
		assertEquals(3, tags2.length);
		
		// Delete the document just created
		docc.delete(sid, docId);
		
		// verify the effective deletion of the document
		try {
			docInfo = docc.getDocument(sid, docId);
			fail("The document " + docInfo.getId() +" should be Deleted!");
		} catch (RuntimeException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
		}
	}

}
